package javax.print;

import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;
import java.security.PrivilegedExceptionAction;
import java.util.Iterator;
import javax.print.attribute.AttributeSet;
import java.util.ArrayList;
import sun.awt.AppContext;

public abstract class PrintServiceLookup
{
    public PrintServiceLookup() {
    }
    
    private static PrintServiceLookup.Services getServicesForContext() {
        PrintServiceLookup.Services value = (PrintServiceLookup.Services)AppContext.getAppContext().get(PrintServiceLookup.Services.class);
        if (value == null) {
            value = new PrintServiceLookup.Services();
            AppContext.getAppContext().put(PrintServiceLookup.Services.class, value);
            return value;
        }
        return value;
    }
    
    private static ArrayList getListOfLookupServices() {
        return getServicesForContext().listOfLookupServices;
    }
    
    private static ArrayList initListOfLookupServices() {
        ArrayList<Object> list = new ArrayList();
        getServicesForContext().listOfLookupServices = list;
        return list;
    }
    
    private static ArrayList getRegisteredServices() {
        return getServicesForContext().registeredServices;
    }
    
    private static ArrayList initRegisteredServices() {
        ArrayList<Object> list = new ArrayList();
        getServicesForContext().registeredServices = list;
        return list;
    }
    
    public static final PrintService[] lookupPrintServices(DocFlavor docFlavor, AttributeSet set) {
        ArrayList<Object> services = getServices(docFlavor, set);
        return (PrintService[])(PrintService[])services.toArray(new PrintService[services.size()]);
    }
    
    public static final MultiDocPrintService[] lookupMultiDocPrintServices(DocFlavor[] array, AttributeSet set) {
        ArrayList<Object> multiDocServices = getMultiDocServices(array, set);
        return (MultiDocPrintService[])(MultiDocPrintService[])multiDocServices.toArray(new MultiDocPrintService[multiDocServices.size()]);
    }
    
    public static final PrintService lookupDefaultPrintService() {
        Iterator<Object> iterator = getAllLookupServices().iterator();
        while (iterator.hasNext()) {
            try {
                PrintService defaultPrintService = ((PrintServiceLookup)iterator.next()).getDefaultPrintService();
                if (defaultPrintService != null) {
                    return defaultPrintService;
                }
                continue;
            }
            catch (Exception ex) {}
        }
        return null;
    }
    
    public static boolean registerServiceProvider(PrintServiceLookup e) {
        synchronized (PrintServiceLookup.class) {
            Iterator<Object> iterator = getAllLookupServices().iterator();
            while (iterator.hasNext()) {
                try {
                    if (iterator.next().getClass() == e.getClass()) {
                        return false;
                    }
                    continue;
                }
                catch (Exception ex) {}
            }
            getListOfLookupServices().add(e);
            return true;
        }
    }
    
    public static boolean registerService(PrintService printService) {
        synchronized (PrintServiceLookup.class) {
            if ((printService instanceof StreamPrintService)) {
                return false;
            }
            ArrayList<PrintService> list = getRegisteredServices();
            if (list == null) {
                list = initRegisteredServices();
            }
            else if (list.contains(printService)) {
                return false;
            }
            list.add(printService);
            return true;
        }
    }
    
    public abstract PrintService[] getPrintServices(DocFlavor p0, AttributeSet p1);
    
    public abstract PrintService[] getPrintServices();
    
    public abstract MultiDocPrintService[] getMultiDocPrintServices(DocFlavor[] p0, AttributeSet p1);
    
    public abstract PrintService getDefaultPrintService();
    
    private static ArrayList getAllLookupServices() {
        synchronized (PrintServiceLookup.class) {
            ArrayList<Object> listOfLookupServices = getListOfLookupServices();
            if (listOfLookupServices != null) {
                return listOfLookupServices;
            }
            ArrayList<Object> initListOfLookupServices = initListOfLookupServices();
            try {
                AccessController.doPrivileged(new PrivilegedExceptionAction() {
                    @Override
                    public Object run() {
                        Iterator<PrintServiceLookup> iterator = ServiceLoader.load(PrintServiceLookup.class).iterator();
                        ArrayList<Object> access$200 = PrintServiceLookup.access$200();
                        while (iterator.hasNext()) {
                            try {
                                access$200.add(iterator.next());
                            }
                            catch (ServiceConfigurationError serviceConfigurationError) {
                                if (System.getSecurityManager() != null) {
                                    serviceConfigurationError.printStackTrace();
                                }
                                else {
                                    throw serviceConfigurationError;
                                }
                            }
                        }
                        return null;
                    }
                });
            }
            catch (PrivilegedActionException ex) {}
            return initListOfLookupServices;
        }
    }
    
    private static ArrayList getServices(DocFlavor docFlavor, AttributeSet set) {
        ArrayList<Object> list = new ArrayList();
        Iterator<Object> iterator = getAllLookupServices().iterator();
        while (iterator.hasNext()) {
            try {
                PrintServiceLookup printServiceLookup = (PrintServiceLookup)iterator.next();
                PrintService[] array = null;
                if (docFlavor == null && set == null) {
                    try {
                        array = printServiceLookup.getPrintServices();
                    }
                    catch (Throwable t) {}
                }
                else {
                    array = printServiceLookup.getPrintServices(docFlavor, set);
                }
                if (array == null) {
                    continue;
                }
                for (int i = 0; i < array.length; ++i) {
                    list.add(array[i]);
                }
            }
            catch (Exception ex) {}
        }
        ArrayList<Object> registeredServices = null;
        try {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPrintJobAccess();
            }
            registeredServices = getRegisteredServices();
        }
        catch (SecurityException ex2) {}
        if (registeredServices != null) {
            PrintService[] array2 = (PrintService[])(PrintService[])registeredServices.toArray(new PrintService[registeredServices.size()]);
            for (int j = 0; j < array2.length; ++j) {
                if (!list.contains(array2[j])) {
                    if (docFlavor == null && set == null) {
                        list.add(array2[j]);
                    }
                    else if (((docFlavor != null && array2[j].isDocFlavorSupported(docFlavor)) || docFlavor == null) && null == array2[j].getUnsupportedAttributes(docFlavor, set)) {
                        list.add(array2[j]);
                    }
                }
            }
            return list;
        }
        return list;
    }
    
    private static ArrayList getMultiDocServices(DocFlavor[] array, AttributeSet set) {
        ArrayList<Object> list = new ArrayList();
        Iterator<Object> iterator = getAllLookupServices().iterator();
        while (iterator.hasNext()) {
            try {
                MultiDocPrintService[] multiDocPrintServices = ((PrintServiceLookup)iterator.next()).getMultiDocPrintServices(array, set);
                if (multiDocPrintServices == null) {
                    continue;
                }
                for (int i = 0; i < multiDocPrintServices.length; ++i) {
                    list.add(multiDocPrintServices[i]);
                }
            }
            catch (Exception ex) {}
        }
        ArrayList<Object> registeredServices = null;
        try {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPrintJobAccess();
            }
            registeredServices = getRegisteredServices();
        }
        catch (Exception ex2) {}
        if (registeredServices != null) {
            PrintService[] array2 = (PrintService[])(PrintService[])registeredServices.toArray(new PrintService[registeredServices.size()]);
            for (int j = 0; j < array2.length; ++j) {
                if (array2[j] instanceof MultiDocPrintService && !list.contains(array2[j])) {
                    if (array == null || array.length == 0) {
                        list.add(array2[j]);
                    }
                    else {
                        boolean n = true;
                        int k = 0;
                        while (k < array.length) {
                            if (array2[j].isDocFlavorSupported(array[k])) {
                                if (array2[j].getUnsupportedAttributes(array[k], set) != null) {
                                    n = false;
                                    break;
                                }
                                ++k;
                            }
                            else {
                                n = false;
                                break;
                            }
                        }
                        if (!(n == 0)) {
                            list.add(array2[j]);
                        }
                    }
                }
            }
            return list;
        }
        return list;
    }
    
    static ArrayList access$200() {
        return getListOfLookupServices();
    }
    
    static class Services
    {
        private ArrayList listOfLookupServices;
        private ArrayList registeredServices;
        
        Services() {
            this.listOfLookupServices = null;
            this.registeredServices = null;
        }
    }
}
